@@ -2,27 +2,28 @@
  *
  * SPDX-License-Identifier: Apache-2.0 */
 
-#ifndef __BAKE_H__
-#define __BAKE_H__
+#pragma once
 
 #include "device/device.h"
 #include "scene/scene.h"
 
 #include "util/progress.h"
-#include "util/vector.h"
 
 CCL_NAMESPACE_BEGIN
 
 class BakeManager {
  public:
-  BakeManager();
-  ~BakeManager();
+  BakeManager() = default;
+  ~BakeManager() = default;
 
-  void set(Scene *scene, const std::string &object_name);
+  void set_baking(Scene *scene, const bool use);
   bool get_baking() const;
 
   void set_use_camera(bool use_camera);
 
+  void set_use_seed(bool use_seed);
+  bool get_use_seed() const;
+
   void device_update(Device *device, DeviceScene *dscene, Scene *scene, Progress &progress);
   void device_free(Device *device, DeviceScene *dscene);
 
@@ -31,11 +32,10 @@ class BakeManager {
   bool need_update() const;
 
  private:
-  bool need_update_;
-  std::string object_name;
-  bool use_camera_;
+  bool need_update_ = true;
+  bool use_baking_ = false;
+  bool use_camera_ = false;
+  bool use_seed_ = false;
 };
 
 CCL_NAMESPACE_END
-
-#endif /* __BAKE_H__ */
