@@ -2,11 +2,10 @@
  *
  * SPDX-License-Identifier: Apache-2.0 */
 
-#ifndef __UTIL_SYSTEM_H__
-#define __UTIL_SYSTEM_H__
+#pragma once
 
-#include <stdint.h>
-#include <stdlib.h>
+#include <cstdint>
+#include <cstdlib>
 
 #include <string>
 
@@ -17,8 +16,7 @@ int system_console_width();
 
 std::string system_cpu_brand_string();
 int system_cpu_bits();
-bool system_cpu_support_sse2();
-bool system_cpu_support_sse41();
+bool system_cpu_support_sse42();
 bool system_cpu_support_avx2();
 
 size_t system_physical_ram();
@@ -27,5 +25,3 @@ size_t system_physical_ram();
 uint64_t system_self_process_id();
 
 CCL_NAMESPACE_END
-
-#endif /* __UTIL_SYSTEM_H__ */
