@@ -3,9 +3,9 @@
  * SPDX-License-Identifier: GPL-2.0-or-later */
 #include "testing/testing.h"
 
-#include "MEM_guardedalloc.h"
+#include "BKE_fcurve.hh"
 
-#include "BKE_fcurve.h"
+#include "ANIM_fcurve.hh"
 
 #include "ED_keyframing.hh"
 
@@ -14,6 +14,7 @@
 #include "BLI_math_vector_types.hh"
 
 namespace blender::bke::tests {
+using namespace blender::animrig;
 
 /* Epsilon for floating point comparisons. */
 static const float EPSILON = 1e-7f;
@@ -29,9 +30,10 @@ TEST(evaluate_fcurve, OnKeys)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 3.0f, 19.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {3.0f, 19.0f}, settings, INSERTKEY_NOFLAGS);
 
   EXPECT_NEAR(evaluate_fcurve(fcu, 1.0f), 7.0f, EPSILON);  /* hits 'on or before first' function */
   EXPECT_NEAR(evaluate_fcurve(fcu, 2.0f), 13.0f, EPSILON); /* hits 'between' function */
@@ -52,8 +54,13 @@ TEST(evaluate_fcurve, InterpolationConstant)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 2.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 13.0f);
 
   fcu->bezt[0].ipo = BEZT_IPO_CONST;
   fcu->bezt[1].ipo = BEZT_IPO_CONST;
@@ -68,8 +75,13 @@ TEST(evaluate_fcurve, InterpolationLinear)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 2.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 13.0f);
 
   fcu->bezt[0].ipo = BEZT_IPO_LIN;
   fcu->bezt[1].ipo = BEZT_IPO_LIN;
@@ -85,8 +97,13 @@ TEST(evaluate_fcurve, InterpolationBezier)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 2.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 13.0f);
 
   EXPECT_EQ(fcu->bezt[0].ipo, BEZT_IPO_BEZ);
   EXPECT_EQ(fcu->bezt[1].ipo, BEZT_IPO_BEZ);
@@ -118,8 +135,13 @@ TEST(evaluate_fcurve, InterpolationBounce)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 2.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 13.0f);
 
   fcu->bezt[0].ipo = BEZT_IPO_BOUNCE;
   fcu->bezt[1].ipo = BEZT_IPO_BOUNCE;
@@ -138,8 +160,14 @@ TEST(evaluate_fcurve, ExtrapolationLinearKeys)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 2.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 13.0f);
+
   fcu->bezt[0].ipo = BEZT_IPO_LIN;
   fcu->bezt[1].ipo = BEZT_IPO_LIN;
 
@@ -167,8 +195,13 @@ TEST(evaluate_fcurve, ExtrapolationBezierKeys)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 2.0f, 13.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {2.0f, 13.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 2.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 13.0f);
 
   fcu->bezt[0].vec[0][0] = 0.71855f; /* left handle X */
   fcu->bezt[0].vec[0][1] = 6.22482f; /* left handle Y */
@@ -203,9 +236,14 @@ TEST(fcurve_subdivide, BKE_fcurve_bezt_subdivide_handles)
 {
   FCurve *fcu = BKE_fcurve_create();
 
+  const KeyframeSettings settings = get_keyframe_settings(false);
   /* Insert two keyframes and set handles to something non-default. */
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 0.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 13.0f, 2.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  insert_vert_fcurve(fcu, {1.0f, 0.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {13.0f, 2.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 0.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 13.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 2.0f);
 
   fcu->bezt[0].h1 = fcu->bezt[0].h2 = HD_FREE;
   fcu->bezt[0].vec[0][0] = -5.0f;
@@ -269,12 +307,19 @@ TEST(fcurve_active_keyframe, ActiveKeyframe)
   /* There should be no active keyframe with no points. */
   EXPECT_EQ(BKE_fcurve_active_keyframe_index(fcu), FCURVE_ACTIVE_KEYFRAME_NONE);
 
+  const KeyframeSettings settings = get_keyframe_settings(false);
   /* Check that adding new points sets the active index. */
-  EXPECT_EQ(insert_vert_fcurve(fcu, 1.0f, 7.5f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 0);
+  insert_vert_fcurve(fcu, {1.0f, 7.5f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[0].vec[1][0], 1.0f);
+  EXPECT_EQ(fcu->bezt[0].vec[1][1], 7.5f);
   EXPECT_EQ(BKE_fcurve_active_keyframe_index(fcu), 0);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 8.0f, 15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 1);
+  insert_vert_fcurve(fcu, {8.0f, 15.0f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[1].vec[1][0], 8.0f);
+  EXPECT_EQ(fcu->bezt[1].vec[1][1], 15.0f);
   EXPECT_EQ(BKE_fcurve_active_keyframe_index(fcu), 1);
-  EXPECT_EQ(insert_vert_fcurve(fcu, 14.0f, 8.2f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF), 2);
+  insert_vert_fcurve(fcu, {14.0f, 8.2f}, settings, INSERTKEY_NOFLAGS);
+  EXPECT_EQ(fcu->bezt[2].vec[1][0], 14.0f);
+  EXPECT_EQ(fcu->bezt[2].vec[1][1], 8.2f);
   EXPECT_EQ(BKE_fcurve_active_keyframe_index(fcu), 2);
 
   /* Check clearing the index. */
@@ -322,9 +367,10 @@ TEST(BKE_fcurve, BKE_fcurve_keyframe_move_value_with_handles)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  insert_vert_fcurve(fcu, 1.0f, 7.5f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 8.0f, 15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 14.0f, 8.2f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.5f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {8.0f, 15.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {14.0f, 8.2f}, settings, INSERTKEY_NOFLAGS);
 
   EXPECT_FLOAT_EQ(fcu->bezt[1].vec[0][0], 5.2671194f);
   EXPECT_FLOAT_EQ(fcu->bezt[1].vec[0][1], 15.0f);
@@ -353,9 +399,10 @@ TEST(BKE_fcurve, BKE_fcurve_keyframe_move_time_with_handles)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  insert_vert_fcurve(fcu, 1.0f, 7.5f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 8.0f, 15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 14.0f, 8.2f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.5f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {8.0f, 15.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {14.0f, 8.2f}, settings, INSERTKEY_NOFLAGS);
 
   EXPECT_FLOAT_EQ(fcu->bezt[1].vec[0][0], 5.2671194f);
   EXPECT_FLOAT_EQ(fcu->bezt[1].vec[0][1], 15.0f);
@@ -384,11 +431,12 @@ TEST(BKE_fcurve, BKE_fcurve_calc_range)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  insert_vert_fcurve(fcu, 1.0f, 7.5f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 4.0f, -15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 8.0f, 15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 14.0f, 8.2f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 18.2f, -20.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.5f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {4.0f, -15.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {8.0f, 15.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {14.0f, 8.2f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {18.2f, -20.0f}, settings, INSERTKEY_NOFLAGS);
 
   for (int i = 0; i < fcu->totvert; i++) {
     fcu->bezt[i].f1 &= ~SELECT;
@@ -436,11 +484,12 @@ TEST(BKE_fcurve, BKE_fcurve_calc_bounds)
 {
   FCurve *fcu = BKE_fcurve_create();
 
-  insert_vert_fcurve(fcu, 1.0f, 7.5f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 4.0f, -15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 8.0f, 15.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 14.0f, 8.2f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
-  insert_vert_fcurve(fcu, 18.2f, -20.0f, BEZT_KEYTYPE_KEYFRAME, INSERTKEY_NO_USERPREF);
+  const KeyframeSettings settings = get_keyframe_settings(false);
+  insert_vert_fcurve(fcu, {1.0f, 7.5f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {4.0f, -15.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {8.0f, 15.0f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {14.0f, 8.2f}, settings, INSERTKEY_NOFLAGS);
+  insert_vert_fcurve(fcu, {18.2f, -20.0f}, settings, INSERTKEY_NOFLAGS);
 
   for (int i = 0; i < fcu->totvert; i++) {
     fcu->bezt[i].f1 &= ~SELECT;
