@@ -20,18 +20,16 @@
 #include "BLI_math_color.h"
 #include "BLI_math_vector.h"
 #include "BLI_string.h"
-#include "BLI_string_utils.h"
+#include "BLI_string_utils.hh"
 #include "BLI_threads.h"
-#include "BLI_utildefines.h"
 
-#include "BLT_translation.h"
+#include "BLT_translation.hh"
 
 #include "BKE_movieclip.h"
 #include "BKE_tracking.h"
 
-#include "IMB_imbuf.h"
-#include "IMB_imbuf_types.h"
-#include "IMB_moviecache.h"
+#include "IMB_imbuf.hh"
+#include "IMB_imbuf_types.hh"
 
 #include "tracking_private.h"
 
@@ -52,13 +50,13 @@
 
 TracksMap *tracks_map_new(const char *object_name, int num_tracks)
 {
-  TracksMap *map = MEM_cnew<TracksMap>("TrackingsMap");
+  TracksMap *map = MEM_callocN<TracksMap>("TrackingsMap");
 
   STRNCPY(map->object_name, object_name);
 
   map->num_tracks = num_tracks;
 
-  map->tracks = MEM_cnew_array<MovieTrackingTrack>(num_tracks, "TrackingsMap tracks");
+  map->tracks = MEM_calloc_arrayN<MovieTrackingTrack>(num_tracks, "TrackingsMap tracks");
 
   map->hash = BLI_ghash_ptr_new("TracksMap hash");
 
@@ -435,7 +433,7 @@ static void distortion_model_parameters_from_tracking(
   }
 
   /* Unknown distortion model, which might be due to opening newer file in older Blender.
-   * Fallback to a known and supported model with 0 distortion. */
+   * Fall back to a known and supported model with 0 distortion. */
   camera_intrinsics_options->distortion_model = LIBMV_DISTORTION_MODEL_POLYNOMIAL;
   camera_intrinsics_options->polynomial_k1 = 0.0;
   camera_intrinsics_options->polynomial_k2 = 0.0;
@@ -495,8 +493,6 @@ void tracking_cameraIntrinscisOptionsFromTracking(
   tracking_principal_point_normalized_to_pixel(
       camera->principal_point, calibration_width, calibration_height, principal_px);
 
-  camera_intrinsics_options->num_threads = BLI_system_thread_count();
-
   camera_intrinsics_options->focal_length = camera->focal;
 
   camera_intrinsics_options->principal_point_x = principal_px[0];
@@ -548,16 +544,14 @@ MovieTrackingMarker *tracking_get_keyframed_marker(MovieTrackingTrack *track,
 
     if ((cur_marker->flag & MARKER_DISABLED) == 0) {
       /* If it'll happen so we didn't find a real keyframe marker,
-       * fallback to the first marker in current tracked segment
-       * as a keyframe.
-       */
+       * fall back to the first marker in current tracked segment
+       * as a keyframe. */
       if (next_marker == nullptr) {
         /* Could happen when trying to get reference marker for the fist
          * one on the segment which isn't surrounded by disabled markers.
          *
          * There's no really good choice here, just use the reference
-         * marker which looks correct..
-         */
+         * marker which looks correct.. */
         if (marker_keyed_fallback == nullptr) {
           marker_keyed_fallback = cur_marker;
         }
@@ -627,7 +621,7 @@ static ImBuf *make_grayscale_ibuf_copy(ImBuf *ibuf)
    */
   const size_t num_pixels = size_t(grayscale->x) * size_t(grayscale->y);
   grayscale->channels = 1;
-  float *rect_float = MEM_cnew_array<float>(num_pixels, "tracking grayscale image");
+  float *rect_float = MEM_calloc_arrayN<float>(num_pixels, "tracking grayscale image");
   if (rect_float != nullptr) {
     IMB_assign_float_buffer(grayscale, rect_float, IB_TAKE_OWNERSHIP);
 
@@ -655,7 +649,7 @@ static ImBuf *float_image_to_ibuf(libmv_FloatImage *float_image)
   ImBuf *ibuf = IMB_allocImBuf(float_image->width, float_image->height, 32, 0);
   size_t num_total_channels = size_t(ibuf->x) * size_t(ibuf->y) * float_image->channels;
   ibuf->channels = float_image->channels;
-  float *rect_float = MEM_cnew_array<float>(num_total_channels, "tracking grayscale image");
+  float *rect_float = MEM_calloc_arrayN<float>(num_total_channels, "tracking grayscale image");
   if (rect_float != nullptr) {
     IMB_assign_float_buffer(ibuf, rect_float, IB_TAKE_OWNERSHIP);
 
@@ -696,7 +690,7 @@ static ImBuf *accessor_get_ibuf(TrackingImageAccessor *accessor,
     clamped_width = min_ii(clamped_width, orig_ibuf->x - clamped_origin_x);
     clamped_height = min_ii(clamped_height, orig_ibuf->y - clamped_origin_y);
 
-    final_ibuf = IMB_allocImBuf(width, height, 32, IB_rectfloat);
+    final_ibuf = IMB_allocImBuf(width, height, 32, IB_float_data);
 
     if (orig_ibuf->float_buffer.data != nullptr) {
       IMB_rectcpy(final_ibuf,
@@ -733,7 +727,7 @@ static ImBuf *accessor_get_ibuf(TrackingImageAccessor *accessor,
      * frames) but on the other hand it bumps the memory usage up.
      */
     BLI_thread_lock(LOCK_MOVIECLIP);
-    IMB_float_from_rect(orig_ibuf);
+    IMB_float_from_byte(orig_ibuf);
     BLI_thread_unlock(LOCK_MOVIECLIP);
     final_ibuf = orig_ibuf;
   }
@@ -742,7 +736,11 @@ static ImBuf *accessor_get_ibuf(TrackingImageAccessor *accessor,
     if (final_ibuf == orig_ibuf) {
       final_ibuf = IMB_dupImBuf(orig_ibuf);
     }
-    IMB_scaleImBuf(final_ibuf, orig_ibuf->x / (1 << downscale), orig_ibuf->y / (1 << downscale));
+    IMB_scale(final_ibuf,
+              orig_ibuf->x / (1 << downscale),
+              orig_ibuf->y / (1 << downscale),
+              IMBScaleFilter::Box,
+              false);
   }
   /* Apply possible transformation. */
   if (transform != nullptr) {
@@ -883,14 +881,14 @@ TrackingImageAccessor *tracking_image_accessor_new(MovieClip *clips[MAX_ACCESSOR
                                                    MovieTrackingTrack **tracks,
                                                    int num_tracks)
 {
-  TrackingImageAccessor *accessor = MEM_cnew<TrackingImageAccessor>("tracking image accessor");
+  TrackingImageAccessor *accessor = MEM_callocN<TrackingImageAccessor>("tracking image accessor");
 
   BLI_assert(num_clips <= MAX_ACCESSOR_CLIP);
 
   memcpy(accessor->clips, clips, num_clips * sizeof(MovieClip *));
   accessor->num_clips = num_clips;
 
-  accessor->tracks = MEM_cnew_array<MovieTrackingTrack *>(num_tracks, "image accessor tracks");
+  accessor->tracks = MEM_calloc_arrayN<MovieTrackingTrack *>(num_tracks, "image accessor tracks");
   memcpy(accessor->tracks, tracks, num_tracks * sizeof(MovieTrackingTrack *));
   accessor->num_tracks = num_tracks;
 
