@@ -6,15 +6,18 @@
  * \ingroup draw
  */
 
+#include "DNA_userdef_types.h"
+
 #include "BKE_paint.hh"
 #include "BKE_paint_bvh.hh"
 
 #include "BLI_math_base.h"
 #include "GPU_compute.hh"
 
+#include "draw_context_private.hh"
+#include "draw_debug.hh"
 #include "draw_defines.hh"
 #include "draw_manager.hh"
-#include "draw_manager_c.hh"
 #include "draw_pass.hh"
 #include "draw_shader.hh"
 
@@ -30,7 +33,7 @@ Manager::~Manager()
   }
 }
 
-void Manager::begin_sync()
+void Manager::begin_sync(Object *object_active)
 {
   /* Add 2 to always have a non-null number even in case of overflow. */
   sync_counter_ = (global_sync_counter_ += 2);
@@ -72,7 +75,7 @@ void Manager::begin_sync()
   attribute_len_ = 0;
   /* TODO(fclem): Resize buffers if too big, but with an hysteresis threshold. */
 
-  object_active = DST.draw_ctx.obact;
+  this->object_active = object_active;
 
   /* Init the 0 resource. */
   resource_handle(float4x4::identity());
@@ -96,7 +99,7 @@ void Manager::sync_layer_attributes()
 
   for (uint32_t id : id_list) {
     if (layer_attributes_buf[count].sync(
-            DST.draw_ctx.scene, DST.draw_ctx.view_layer, layer_attributes.lookup(id)))
+            drw_get().scene, drw_get().view_layer, layer_attributes.lookup(id)))
     {
       /* Check if the buffer is full. */
       if (++count == size) {
@@ -119,12 +122,13 @@ void Manager::end_sync()
   infos_buf.current().push_update();
   attributes_buf.push_update();
   layer_attributes_buf.push_update();
-  attributes_buf_legacy.push_update();
 
   /* Useful for debugging the following resource finalize. But will trigger the drawing of the GPU
    * debug draw/print buffers for every frame. Not nice for performance. */
   // debug_bind();
 
+  DRW_submission_start();
+
   /* Dispatch compute to finalize the resources on GPU. Save a bit of CPU time. */
   uint thread_groups = divide_ceil_u(resource_len_, DRW_FINALIZE_GROUP_SIZE);
   GPUShader *shader = DRW_shader_draw_resource_finalize_get();
@@ -136,20 +140,21 @@ void Manager::end_sync()
   GPU_compute_dispatch(shader, thread_groups, 1, 1);
   GPU_memory_barrier(GPU_BARRIER_SHADER_STORAGE);
 
+  DRW_submission_end();
+
   GPU_debug_group_end();
 }
 
 void Manager::debug_bind()
 {
-#ifdef WITH_DRAW_DEBUG
-  if (DST.debug == nullptr) {
+  GPUStorageBuf *gpu_buf = DebugDraw::get().gpu_draw_buf_get();
+  if (gpu_buf == nullptr) {
     return;
   }
-  GPU_storagebuf_bind(drw_debug_gpu_draw_buf_get(), DRW_DEBUG_DRAW_SLOT);
-#  ifndef DISABLE_DEBUG_SHADER_PRINT_BARRIER
+  GPU_storagebuf_bind(gpu_buf, DRW_DEBUG_DRAW_SLOT);
+#ifndef DISABLE_DEBUG_SHADER_PRINT_BARRIER
   /* Add a barrier to allow multiple shader writing to the same buffer. */
   GPU_memory_barrier(GPU_BARRIER_SHADER_STORAGE);
-#  endif
 #endif
 }
 
@@ -159,9 +164,6 @@ void Manager::resource_bind()
   GPU_storagebuf_bind(infos_buf.current(), DRW_OBJ_INFOS_SLOT);
   GPU_storagebuf_bind(attributes_buf, DRW_OBJ_ATTR_SLOT);
   GPU_uniformbuf_bind(layer_attributes_buf, DRW_LAYER_ATTR_UBO_SLOT);
-  /* 2 is the hardcoded location of the uniform attr UBO. */
-  /* TODO(@fclem): Remove this workaround. */
-  GPU_uniformbuf_bind(attributes_buf_legacy, 2);
 }
 
 uint64_t Manager::fingerprint_get()
@@ -182,8 +184,8 @@ ResourceHandleRange Manager::resource_handle_for_sculpt(const ObjectRef &ref)
 
 void Manager::compute_visibility(View &view)
 {
-  bool freeze_culling = (USER_EXPERIMENTAL_TEST(&U, use_viewport_debug) && DST.draw_ctx.v3d &&
-                         (DST.draw_ctx.v3d->debug_flag & V3D_DEBUG_FREEZE_CULLING) != 0);
+  bool freeze_culling = (USER_EXPERIMENTAL_TEST(&U, use_viewport_debug) && drw_get().v3d &&
+                         (drw_get().v3d->debug_flag & V3D_DEBUG_FREEZE_CULLING) != 0);
 
   BLI_assert_msg(view.manager_fingerprint_ != this->fingerprint_get(),
                  "Resources did not changed, no need to update");
@@ -204,6 +206,10 @@ void Manager::ensure_visibility(View &view)
 
 void Manager::generate_commands(PassMain &pass, View &view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   BLI_assert_msg((pass.manager_fingerprint_ != this->fingerprint_get()) ||
                      (pass.view_fingerprint_ != view.fingerprint_get()),
                  "Resources and view did not changed no need to update");
@@ -224,12 +230,20 @@ void Manager::generate_commands(PassMain &pass, View &view)
 
 void Manager::generate_commands(PassSortable &pass, View &view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   pass.sort();
   generate_commands(static_cast<PassMain &>(pass), view);
 }
 
 void Manager::generate_commands(PassSimple &pass)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   BLI_assert_msg(pass.manager_fingerprint_ != this->fingerprint_get(),
                  "Resources did not changed since last generate_command, no need to update");
   pass.manager_fingerprint_ = this->fingerprint_get();
@@ -237,8 +251,30 @@ void Manager::generate_commands(PassSimple &pass)
   pass.draw_commands_buf_.generate_commands(pass.headers_, pass.commands_, pass.sub_passes_);
 }
 
+void Manager::warm_shader_specialization(PassMain &pass)
+{
+  if (pass.is_empty()) {
+    return;
+  }
+  command::RecordingState state;
+  pass.warm_shader_specialization(state);
+}
+
+void Manager::warm_shader_specialization(PassSimple &pass)
+{
+  if (pass.is_empty()) {
+    return;
+  }
+  command::RecordingState state;
+  pass.warm_shader_specialization(state);
+}
+
 void Manager::submit_only(PassMain &pass, View &view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   BLI_assert_msg(view.manager_fingerprint_ != 0, "compute_visibility was not called on this view");
   BLI_assert_msg(view.manager_fingerprint_ == this->fingerprint_get(),
                  "Resources changed since last compute_visibility");
@@ -268,6 +304,10 @@ void Manager::submit_only(PassMain &pass, View &view)
 
 void Manager::submit(PassMain &pass, View &view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   if (view.manager_fingerprint_ != this->fingerprint_get()) {
     compute_visibility(view);
   }
@@ -283,6 +323,10 @@ void Manager::submit(PassMain &pass, View &view)
 
 void Manager::submit(PassSortable &pass, View &view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   pass.sort();
 
   this->submit(static_cast<PassMain &>(pass), view);
@@ -290,6 +334,10 @@ void Manager::submit(PassSortable &pass, View &view)
 
 void Manager::submit(PassSimple &pass, bool inverted_view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   if (!pass.has_generated_commands()) {
     generate_commands(pass);
   }
@@ -310,6 +358,10 @@ void Manager::submit(PassSimple &pass, bool inverted_view)
 
 void Manager::submit(PassSimple &pass, View &view)
 {
+  if (pass.is_empty()) {
+    return;
+  }
+
   debug_bind();
 
   view.bind();
