@@ -15,45 +15,49 @@
 #include "BLI_math_vector.h"
 #include "BLI_rect.h"
 
-#include "BKE_action.h"
-#include "BKE_context.h"
-#include "BKE_global.h"
-#include "BKE_gpencil_modifier_legacy.h"
-#include "BKE_idprop.h"
-#include "BKE_layer.h"
-#include "BKE_lib_id.h"
-#include "BKE_main.h"
-#include "BKE_modifier.h"
+#include "BKE_action.hh"
+#include "BKE_context.hh"
+#ifdef WITH_XR_OPENXR
+#  include "BKE_idprop.hh"
+#endif
+#include "BKE_global.hh"
+#include "BKE_layer.hh"
+#include "BKE_lib_id.hh"
+#include "BKE_main.hh"
+#include "BKE_modifier.hh"
 #include "BKE_object.hh"
-#include "BKE_report.h"
-#include "BKE_scene.h"
+#include "BKE_report.hh"
+#include "BKE_scene.hh"
 
 #include "DEG_depsgraph_query.hh"
 
 #include "UI_resources.hh"
 
-#include "GPU_matrix.h"
-#include "GPU_select.h"
-#include "GPU_state.h"
+#include "GPU_matrix.hh"
+#include "GPU_select.hh"
+#include "GPU_state.hh"
 
 #include "WM_api.hh"
 
+#include "ED_info.hh"
 #include "ED_object.hh"
 #include "ED_screen.hh"
 
-#include "DRW_engine.h"
+#include "DRW_engine.hh"
 
 #include "RNA_access.hh"
 #include "RNA_define.hh"
 
-#include "view3d_intern.h" /* own include */
+#include "view3d_intern.hh" /* own include */
 #include "view3d_navigate.hh"
 
+#include "DNA_camera_types.h"
+
 /* -------------------------------------------------------------------- */
 /** \name Camera to View Operator
  * \{ */
 
-static int view3d_camera_to_view_exec(bContext *C, wmOperator * /*op*/)
+static wmOperatorStatus view3d_camera_to_view_exec(bContext *C, wmOperator * /*op*/)
 {
   const Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   View3D *v3d;
@@ -65,6 +69,8 @@ static int view3d_camera_to_view_exec(bContext *C, wmOperator * /*op*/)
   ED_view3d_context_user_region(C, &v3d, &region);
   rv3d = static_cast<RegionView3D *>(region->regiondata);
 
+  ED_view3d_smooth_view_force_finish(C, v3d, region);
+
   ED_view3d_lastview_store(rv3d);
 
   BKE_object_tfm_protected_backup(v3d->camera, &obtfm);
@@ -107,7 +113,7 @@ void VIEW3D_OT_camera_to_view(wmOperatorType *ot)
   ot->description = "Set camera view to active view";
   ot->idname = "VIEW3D_OT_camera_to_view";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view3d_camera_to_view_exec;
   ot->poll = view3d_camera_to_view_poll;
 
@@ -121,9 +127,11 @@ void VIEW3D_OT_camera_to_view(wmOperatorType *ot)
 /** \name Camera Fit Frame to Selected Operator
  * \{ */
 
-/* unlike VIEW3D_OT_view_selected this is for framing a render and not
- * meant to take into account vertex/bone selection for eg. */
-static int view3d_camera_to_view_selected_exec(bContext *C, wmOperator *op)
+/**
+ * Unlike #VIEW3D_OT_view_selected this is for framing a render and not
+ * meant to take into account vertex/bone selection for eg.
+ */
+static wmOperatorStatus view3d_camera_to_view_selected_exec(bContext *C, wmOperator *op)
 {
   Main *bmain = CTX_data_main(C);
   Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
@@ -150,7 +158,7 @@ void VIEW3D_OT_camera_to_view_selected(wmOperatorType *ot)
   ot->description = "Move the camera so selected objects are framed";
   ot->idname = "VIEW3D_OT_camera_to_view_selected";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view3d_camera_to_view_selected_exec;
   ot->poll = ED_operator_scene_editable;
 
@@ -220,7 +228,7 @@ static void sync_viewport_camera_smoothview(bContext *C,
   }
 }
 
-static int view3d_setobjectascamera_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus view3d_setobjectascamera_exec(bContext *C, wmOperator *op)
 {
   View3D *v3d;
   ARegion *region;
@@ -235,13 +243,15 @@ static int view3d_setobjectascamera_exec(bContext *C, wmOperator *op)
   ED_view3d_context_user_region(C, &v3d, &region);
   rv3d = static_cast<RegionView3D *>(region->regiondata);
 
+  ED_view3d_smooth_view_force_finish(C, v3d, region);
+
   if (ob) {
     Object *camera_old = (rv3d->persp == RV3D_CAMOB) ? V3D_CAMERA_SCENE(scene, v3d) : nullptr;
     rv3d->persp = RV3D_CAMOB;
     v3d->camera = ob;
     if (v3d->scenelock && scene->camera != ob) {
       scene->camera = ob;
-      DEG_id_tag_update(&scene->id, ID_RECALC_COPY_ON_WRITE);
+      DEG_id_tag_update(&scene->id, ID_RECALC_SYNC_TO_EVAL);
       DEG_relations_tag_update(CTX_data_main(C));
     }
 
@@ -286,7 +296,7 @@ void VIEW3D_OT_object_as_camera(wmOperatorType *ot)
   ot->description = "Set the active object as the active camera for this view or scene";
   ot->idname = "VIEW3D_OT_object_as_camera";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = view3d_setobjectascamera_exec;
   ot->poll = ED_operator_rv3d_user_region_poll;
 
@@ -300,7 +310,7 @@ void VIEW3D_OT_object_as_camera(wmOperatorType *ot)
 /** \name Window and View Matrix Calculation
  * \{ */
 
-void view3d_winmatrix_set(Depsgraph *depsgraph,
+void view3d_winmatrix_set(const Depsgraph *depsgraph,
                           ARegion *region,
                           const View3D *v3d,
                           const rcti *rect)
@@ -370,14 +380,14 @@ static void obmat_to_viewmat(RegionView3D *rv3d, Object *ob)
 
   rv3d->view = RV3D_VIEW_USER; /* don't show the grid */
 
-  normalize_m4_m4(bmat, ob->object_to_world);
+  normalize_m4_m4(bmat, ob->object_to_world().ptr());
   invert_m4_m4(rv3d->viewmat, bmat);
 
   /* view quat calculation, needed for add object */
   mat4_normalized_to_quat(rv3d->viewquat, rv3d->viewmat);
 }
 
-void view3d_viewmatrix_set(Depsgraph *depsgraph,
+void view3d_viewmatrix_set(const Depsgraph *depsgraph,
                            const Scene *scene,
                            const View3D *v3d,
                            RegionView3D *rv3d,
@@ -385,7 +395,7 @@ void view3d_viewmatrix_set(Depsgraph *depsgraph,
 {
   if (rv3d->persp == RV3D_CAMOB) { /* obs/camera */
     if (v3d->camera) {
-      Object *ob_camera_eval = DEG_get_evaluated_object(depsgraph, v3d->camera);
+      Object *ob_camera_eval = DEG_get_evaluated(depsgraph, v3d->camera);
       obmat_to_viewmat(rv3d, ob_camera_eval);
     }
     else {
@@ -406,15 +416,15 @@ void view3d_viewmatrix_set(Depsgraph *depsgraph,
       rv3d->viewmat[3][2] -= rv3d->dist;
     }
     if (v3d->ob_center) {
-      Object *ob_eval = DEG_get_evaluated_object(depsgraph, v3d->ob_center);
+      Object *ob_eval = DEG_get_evaluated(depsgraph, v3d->ob_center);
       float vec[3];
 
-      copy_v3_v3(vec, ob_eval->object_to_world[3]);
+      copy_v3_v3(vec, ob_eval->object_to_world().location());
       if (ob_eval->type == OB_ARMATURE && v3d->ob_center_bone[0]) {
         bPoseChannel *pchan = BKE_pose_channel_find_name(ob_eval->pose, v3d->ob_center_bone);
         if (pchan) {
           copy_v3_v3(vec, pchan->pose_mat[3]);
-          mul_m4_v3(ob_eval->object_to_world, vec);
+          mul_m4_v3(ob_eval->object_to_world().ptr(), vec);
         }
       }
       translate_m4(rv3d->viewmat, -vec[0], -vec[1], -vec[2]);
@@ -444,9 +454,9 @@ void view3d_viewmatrix_set(Depsgraph *depsgraph,
       vec[2] = 0.0f;
 
       if (rect_scale) {
-        /* Since 'RegionView3D.winmat' has been calculated and this function doesn't take the
-         * 'ARegion' we don't know about the region size.
-         * Use 'rect_scale' when drawing a sub-region to apply 2D offset,
+        /* Since `RegionView3D.winmat` has been calculated and this function doesn't take the
+         * #ARegion we don't know about the region size.
+         * Use `rect_scale` when drawing a sub-region to apply 2D offset,
          * scaled by the difference between the sub-region and the region size.
          */
         vec[0] /= rect_scale[0];
@@ -463,15 +473,15 @@ void view3d_viewmatrix_set(Depsgraph *depsgraph,
 /** \} */
 
 /* -------------------------------------------------------------------- */
-/** \name OpenGL Select Utilities
+/** \name GPU Select Utilities
  * \{ */
 
-void view3d_opengl_select_cache_begin()
+void view3d_gpu_select_cache_begin()
 {
   GPU_select_cache_begin();
 }
 
-void view3d_opengl_select_cache_end()
+void view3d_gpu_select_cache_end()
 {
   GPU_select_cache_end();
 }
@@ -479,8 +489,7 @@ void view3d_opengl_select_cache_end()
 struct DrawSelectLoopUserData {
   uint pass;
   uint hits;
-  GPUSelectResult *buffer;
-  uint buffer_len;
+  GPUSelectBuffer *buffer;
   const rcti *rect;
   eGPUSelectMode gpu_select_mode;
 };
@@ -490,8 +499,7 @@ static bool drw_select_loop_pass(eDRWSelectStage stage, void *user_data)
   bool continue_pass = false;
   DrawSelectLoopUserData *data = static_cast<DrawSelectLoopUserData *>(user_data);
   if (stage == DRW_SELECT_PASS_PRE) {
-    GPU_select_begin_next(
-        data->buffer, data->buffer_len, data->rect, data->gpu_select_mode, data->hits);
+    GPU_select_begin_next(data->buffer, data->rect, data->gpu_select_mode, data->hits);
     /* always run POST after PRE. */
     continue_pass = true;
   }
@@ -501,10 +509,6 @@ static bool drw_select_loop_pass(eDRWSelectStage stage, void *user_data)
       /* quirk of GPU_select_end, only take hits value from first call. */
       data->hits = hits;
     }
-    if (data->gpu_select_mode == GPU_SELECT_NEAREST_FIRST_PASS) {
-      data->gpu_select_mode = GPU_SELECT_NEAREST_SECOND_PASS;
-      continue_pass = (hits > 0);
-    }
     data->pass += 1;
   }
   else {
@@ -540,16 +544,15 @@ static bool drw_select_filter_object_mode_lock(Object *ob, void *user_data)
 static bool drw_select_filter_object_mode_lock_for_weight_paint(Object *ob, void *user_data)
 {
   LinkNode *ob_pose_list = static_cast<LinkNode *>(user_data);
-  return ob_pose_list && (BLI_linklist_index(ob_pose_list, DEG_get_original_object(ob)) != -1);
+  return ob_pose_list && (BLI_linklist_index(ob_pose_list, DEG_get_original(ob)) != -1);
 }
 
-int view3d_opengl_select_ex(ViewContext *vc,
-                            GPUSelectResult *buffer,
-                            uint buffer_len,
-                            const rcti *input,
-                            eV3DSelectMode select_mode,
-                            eV3DSelectObjectFilter select_filter,
-                            const bool do_material_slot_selection)
+int view3d_gpu_select_ex(const ViewContext *vc,
+                         GPUSelectBuffer *buffer,
+                         const rcti *input,
+                         eV3DSelectMode select_mode,
+                         eV3DSelectObjectFilter select_filter,
+                         const bool do_material_slot_selection)
 {
   bThemeState theme_state;
   const wmWindowManager *wm = CTX_wm_manager(vc->C);
@@ -562,13 +565,10 @@ int view3d_opengl_select_ex(ViewContext *vc,
   BKE_view_layer_synced_ensure(scene, vc->view_layer);
   const bool use_obedit_skip = (BKE_view_layer_edit_object_get(vc->view_layer) != nullptr) &&
                                (vc->obedit == nullptr);
-  const bool is_pick_select = (U.gpu_flag & USER_GPU_FLAG_NO_DEPT_PICK) == 0;
-  const bool do_passes = ((is_pick_select == false) &&
-                          (select_mode == VIEW3D_SELECT_PICK_NEAREST));
-  const bool use_nearest = (is_pick_select && select_mode == VIEW3D_SELECT_PICK_NEAREST);
+  const bool use_nearest = select_mode == VIEW3D_SELECT_PICK_NEAREST;
   bool draw_surface = true;
 
-  eGPUSelectMode gpu_select_mode;
+  eGPUSelectMode gpu_select_mode = GPU_SELECT_INVALID;
 
   /* case not a box select */
   if (input->xmin == input->xmax) {
@@ -580,27 +580,17 @@ int view3d_opengl_select_ex(ViewContext *vc,
     rect = *input;
   }
 
-  if (is_pick_select) {
-    if (select_mode == VIEW3D_SELECT_PICK_NEAREST) {
-      gpu_select_mode = GPU_SELECT_PICK_NEAREST;
-    }
-    else if (select_mode == VIEW3D_SELECT_PICK_ALL) {
-      gpu_select_mode = GPU_SELECT_PICK_ALL;
-    }
-    else {
-      gpu_select_mode = GPU_SELECT_ALL;
-    }
+  if (select_mode == VIEW3D_SELECT_PICK_NEAREST) {
+    gpu_select_mode = GPU_SELECT_PICK_NEAREST;
+  }
+  else if (select_mode == VIEW3D_SELECT_PICK_ALL) {
+    gpu_select_mode = GPU_SELECT_PICK_ALL;
   }
   else {
-    if (do_passes) {
-      gpu_select_mode = GPU_SELECT_NEAREST_FIRST_PASS;
-    }
-    else {
-      gpu_select_mode = GPU_SELECT_ALL;
-    }
+    gpu_select_mode = GPU_SELECT_ALL;
   }
 
-  /* Important to use 'vc->obact', not 'BKE_view_layer_active_object_get(vc->view_layer)' below,
+  /* Important to use `vc->obact`, not `BKE_view_layer_active_object_get(vc->view_layer)` below,
    * so it will be nullptr when hidden. */
   struct {
     DRW_ObjectFilterFn fn;
@@ -610,10 +600,10 @@ int view3d_opengl_select_ex(ViewContext *vc,
   /* Re-use cache (rect must be smaller than the cached)
    * other context is assumed to be unchanged */
   if (GPU_select_is_cached()) {
-    GPU_select_begin_next(buffer, buffer_len, &rect, gpu_select_mode, 0);
+    GPU_select_begin_next(buffer, &rect, gpu_select_mode, 0);
     GPU_select_cache_load_id();
     hits = GPU_select_end();
-    goto finally;
+    return hits;
   }
 
   switch (select_filter) {
@@ -628,32 +618,23 @@ int view3d_opengl_select_ex(ViewContext *vc,
     case VIEW3D_SELECT_FILTER_WPAINT_POSE_MODE_LOCK: {
       Object *obact = vc->obact;
       BLI_assert(obact && (obact->mode & OB_MODE_ALL_WEIGHT_PAINT));
-      /* While this uses 'alloca' in a loop (which we typically avoid),
+      /* While this uses `alloca` in a loop (which we typically avoid),
        * the number of items is nearly always 1, maybe 2..3 in rare cases. */
       LinkNode *ob_pose_list = nullptr;
-      if (obact->type == OB_GPENCIL_LEGACY) {
-        GpencilVirtualModifierData virtual_modifier_data;
-        const GpencilModifierData *md = BKE_gpencil_modifiers_get_virtual_modifierlist(
-            obact, &virtual_modifier_data);
-        for (; md; md = md->next) {
-          if (md->type == eGpencilModifierType_Armature) {
-            ArmatureGpencilModifierData *agmd = (ArmatureGpencilModifierData *)md;
-            if (agmd->object && (agmd->object->mode & OB_MODE_POSE)) {
-              BLI_linklist_prepend_alloca(&ob_pose_list, agmd->object);
-            }
+      VirtualModifierData virtual_modifier_data;
+      ModifierData *md = BKE_modifiers_get_virtual_modifierlist(obact, &virtual_modifier_data);
+      for (; md; md = md->next) {
+        if (md->type == eModifierType_Armature) {
+          ArmatureModifierData *amd = reinterpret_cast<ArmatureModifierData *>(md);
+          if (amd->object && (amd->object->mode & OB_MODE_POSE)) {
+            BLI_linklist_prepend_alloca(&ob_pose_list, amd->object);
           }
         }
-      }
-      else {
-        VirtualModifierData virtual_modifier_data;
-        const ModifierData *md = BKE_modifiers_get_virtual_modifierlist(obact,
-                                                                        &virtual_modifier_data);
-        for (; md; md = md->next) {
-          if (md->type == eModifierType_Armature) {
-            ArmatureModifierData *amd = (ArmatureModifierData *)md;
-            if (amd->object && (amd->object->mode & OB_MODE_POSE)) {
-              BLI_linklist_prepend_alloca(&ob_pose_list, amd->object);
-            }
+        else if (md->type == eModifierType_GreasePencilArmature) {
+          GreasePencilArmatureModifierData *amd =
+              reinterpret_cast<GreasePencilArmatureModifierData *>(md);
+          if (amd->object && (amd->object->mode & OB_MODE_POSE)) {
+            BLI_linklist_prepend_alloca(&ob_pose_list, amd->object);
           }
         }
       }
@@ -686,12 +667,11 @@ int view3d_opengl_select_ex(ViewContext *vc,
   /* If in X-ray mode, we select the wires in priority. */
   if (XRAY_ACTIVE(v3d) && use_nearest) {
     /* We need to call "GPU_select_*" API's inside DRW_draw_select_loop
-     * because the OpenGL context created & destroyed inside this function. */
+     * because the GPU context created & destroyed inside this function. */
     DrawSelectLoopUserData drw_select_loop_user_data = {};
     drw_select_loop_user_data.pass = 0;
     drw_select_loop_user_data.hits = 0;
     drw_select_loop_user_data.buffer = buffer;
-    drw_select_loop_user_data.buffer_len = buffer_len;
     drw_select_loop_user_data.rect = &rect;
     drw_select_loop_user_data.gpu_select_mode = gpu_select_mode;
 
@@ -716,12 +696,11 @@ int view3d_opengl_select_ex(ViewContext *vc,
 
   if (hits == 0) {
     /* We need to call "GPU_select_*" API's inside DRW_draw_select_loop
-     * because the OpenGL context created & destroyed inside this function. */
+     * because the GPU context created & destroyed inside this function. */
     DrawSelectLoopUserData drw_select_loop_user_data = {};
     drw_select_loop_user_data.pass = 0;
     drw_select_loop_user_data.hits = 0;
     drw_select_loop_user_data.buffer = buffer;
-    drw_select_loop_user_data.buffer_len = buffer_len;
     drw_select_loop_user_data.rect = &rect;
     drw_select_loop_user_data.gpu_select_mode = gpu_select_mode;
 
@@ -754,39 +733,36 @@ int view3d_opengl_select_ex(ViewContext *vc,
 
   UI_Theme_Restore(&theme_state);
 
-finally:
-
-  if (hits < 0) {
-    printf("Too many objects in select buffer\n"); /* XXX make error message */
-  }
-
   return hits;
 }
 
-int view3d_opengl_select(ViewContext *vc,
-                         GPUSelectResult *buffer,
-                         uint buffer_len,
-                         const rcti *input,
-                         eV3DSelectMode select_mode,
-                         eV3DSelectObjectFilter select_filter)
+int view3d_gpu_select(const ViewContext *vc,
+                      GPUSelectBuffer *buffer,
+                      const rcti *input,
+                      eV3DSelectMode select_mode,
+                      eV3DSelectObjectFilter select_filter)
 {
-  return view3d_opengl_select_ex(vc, buffer, buffer_len, input, select_mode, select_filter, false);
+  return view3d_gpu_select_ex(vc, buffer, input, select_mode, select_filter, false);
 }
 
-int view3d_opengl_select_with_id_filter(ViewContext *vc,
-                                        GPUSelectResult *buffer,
-                                        const uint buffer_len,
-                                        const rcti *input,
-                                        eV3DSelectMode select_mode,
-                                        eV3DSelectObjectFilter select_filter,
-                                        uint select_id)
+int view3d_gpu_select_with_id_filter(const ViewContext *vc,
+                                     GPUSelectBuffer *buffer,
+                                     const rcti *input,
+                                     eV3DSelectMode select_mode,
+                                     eV3DSelectObjectFilter select_filter,
+                                     uint select_id)
 {
-  int hits = view3d_opengl_select(vc, buffer, buffer_len, input, select_mode, select_filter);
+  const int64_t start = buffer->storage.size();
+  int hits = view3d_gpu_select(vc, buffer, input, select_mode, select_filter);
 
   /* Selection sometimes uses -1 for an invalid selection ID, remove these as they
    * interfere with detection of actual number of hits in the selection. */
   if (hits > 0) {
-    hits = GPU_select_buffer_remove_by_id(buffer, hits, select_id);
+    hits = GPU_select_buffer_remove_by_id(buffer->storage.as_mutable_span().slice(start, hits),
+                                          select_id);
+
+    /* Trim buffer to the exact size in case selections were removed. */
+    buffer->storage.resize(start + hits);
   }
   return hits;
 }
@@ -810,7 +786,7 @@ static uint free_localview_bit(Main *bmain)
         if (sl->spacetype == SPACE_VIEW3D) {
           View3D *v3d = reinterpret_cast<View3D *>(sl);
           if (v3d->localvd) {
-            local_view_bits |= v3d->local_view_uuid;
+            local_view_bits |= v3d->local_view_uid;
           }
         }
       }
@@ -838,13 +814,13 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
                                   ReportList *reports)
 {
   View3D *v3d = static_cast<View3D *>(area->spacedata.first);
-  float min[3], max[3], box[3];
+  blender::float3 min, max, box;
   float size = 0.0f;
   uint local_view_bit;
-  bool ok = false;
+  bool changed = false;
 
   if (v3d->localvd) {
-    return ok;
+    return changed;
   }
 
   INIT_MINMAX(min, max);
@@ -855,7 +831,7 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
     /* TODO(dfelinto): We can kick one of the other 3D views out of local view
      * specially if it is not being used. */
     BKE_report(reports, RPT_ERROR, "No more than 16 local views");
-    ok = false;
+    changed = false;
   }
   else {
     BKE_view_layer_synced_ensure(scene, view_layer);
@@ -866,9 +842,10 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
         base->local_view_bits &= ~local_view_bit;
       }
       FOREACH_BASE_IN_EDIT_MODE_BEGIN (scene, view_layer, v3d, base_iter) {
-        BKE_object_minmax(base_iter->object, min, max, false);
+        Object *ob_eval = DEG_get_evaluated(depsgraph, base_iter->object);
+        BKE_object_minmax(ob_eval ? ob_eval : base_iter->object, min, max);
         base_iter->local_view_bits |= local_view_bit;
-        ok = true;
+        changed = true;
       }
       FOREACH_BASE_IN_EDIT_MODE_END;
     }
@@ -876,9 +853,10 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
       BKE_view_layer_synced_ensure(scene, view_layer);
       LISTBASE_FOREACH (Base *, base, BKE_view_layer_object_bases_get(view_layer)) {
         if (BASE_SELECTED(v3d, base)) {
-          BKE_object_minmax(base->object, min, max, false);
+          Object *ob_eval = DEG_get_evaluated(depsgraph, base->object);
+          BKE_object_minmax(ob_eval ? ob_eval : base->object, min, max);
           base->local_view_bits |= local_view_bit;
-          ok = true;
+          changed = true;
         }
         else {
           base->local_view_bits &= ~local_view_bit;
@@ -890,13 +868,23 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
     size = max_fff(box[0], box[1], box[2]);
   }
 
-  if (ok == false) {
+  if (changed == false) {
     return false;
   }
 
-  v3d->localvd = static_cast<View3D *>(MEM_mallocN(sizeof(View3D), "localview"));
+  /* Apply any running smooth-view values before reading from the viewport. */
+  LISTBASE_FOREACH (ARegion *, region, &area->regionbase) {
+    if (region->regiontype == RGN_TYPE_WINDOW) {
+      RegionView3D *rv3d = static_cast<RegionView3D *>(region->regiondata);
+      if (rv3d->sms) {
+        ED_view3d_smooth_view_force_finish_no_camera_lock(depsgraph, wm, win, scene, v3d, region);
+      }
+    }
+  }
+
+  v3d->localvd = MEM_mallocN<View3D>("localview");
   *v3d->localvd = blender::dna::shallow_copy(*v3d);
-  v3d->local_view_uuid = local_view_bit;
+  v3d->local_view_uid = local_view_bit;
 
   LISTBASE_FOREACH (ARegion *, region, &area->regionbase) {
     if (region->regiontype == RGN_TYPE_WINDOW) {
@@ -907,8 +895,7 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
       Object *camera_old = nullptr;
       float dist_new, ofs_new[3];
 
-      rv3d->localvd = static_cast<RegionView3D *>(
-          MEM_mallocN(sizeof(RegionView3D), "localview region"));
+      rv3d->localvd = MEM_mallocN<RegionView3D>("localview region");
       memcpy(rv3d->localvd, rv3d, sizeof(RegionView3D));
 
       if (frame_selected) {
@@ -917,8 +904,9 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
         negate_v3_v3(ofs_new, mid);
 
         if (rv3d->persp == RV3D_CAMOB) {
-          rv3d->persp = RV3D_PERSP;
           camera_old = v3d->camera;
+          const Camera &camera = *static_cast<Camera *>(camera_old->data);
+          rv3d->persp = (camera.type == CAM_ORTHO) ? RV3D_ORTHO : RV3D_PERSP;
         }
 
         if (rv3d->persp == RV3D_ORTHO) {
@@ -933,7 +921,8 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
 
           if (rv3d->persp == RV3D_PERSP) {
             /* Don't zoom closer than the near clipping plane. */
-            dist_new = max_ff(dist_new, v3d->clip_start * 1.5f);
+            const float dist_min = ED_view3d_dist_soft_min_get(v3d, true);
+            CLAMP_MIN(dist_new, dist_min);
           }
         }
 
@@ -952,10 +941,10 @@ static bool view3d_localview_init(const Depsgraph *depsgraph,
     }
   }
 
-  return ok;
+  return changed;
 }
 
-static void view3d_localview_exit(const Depsgraph *depsgraph,
+static bool view3d_localview_exit(const Depsgraph *depsgraph,
                                   wmWindowManager *wm,
                                   wmWindow *win,
                                   const Scene *scene,
@@ -965,26 +954,40 @@ static void view3d_localview_exit(const Depsgraph *depsgraph,
                                   const int smooth_viewtx)
 {
   View3D *v3d = static_cast<View3D *>(area->spacedata.first);
+  bool changed = false;
 
   if (v3d->localvd == nullptr) {
-    return;
+    return changed;
   }
   BKE_view_layer_synced_ensure(scene, view_layer);
   LISTBASE_FOREACH (Base *, base, BKE_view_layer_object_bases_get(view_layer)) {
-    if (base->local_view_bits & v3d->local_view_uuid) {
-      base->local_view_bits &= ~v3d->local_view_uuid;
+    if (base->local_view_bits & v3d->local_view_uid) {
+      base->local_view_bits &= ~v3d->local_view_uid;
+    }
+  }
+
+  /* Apply any running smooth-view values before reading from the viewport. */
+  LISTBASE_FOREACH (ARegion *, region, &area->regionbase) {
+    if (region->regiontype == RGN_TYPE_WINDOW) {
+      RegionView3D *rv3d = static_cast<RegionView3D *>(region->regiondata);
+      if (rv3d->localvd == nullptr) {
+        continue;
+      }
+      if (rv3d->sms) {
+        ED_view3d_smooth_view_force_finish_no_camera_lock(depsgraph, wm, win, scene, v3d, region);
+      }
     }
   }
 
   Object *camera_old = v3d->camera;
   Object *camera_new = v3d->localvd->camera;
 
-  v3d->local_view_uuid = 0;
+  v3d->local_view_uid = 0;
   v3d->camera = v3d->localvd->camera;
 
   MEM_freeN(v3d->localvd);
   v3d->localvd = nullptr;
-  MEM_SAFE_FREE(v3d->runtime.local_stats);
+  ED_view3d_local_stats_free(v3d);
 
   LISTBASE_FOREACH (ARegion *, region, &area->regionbase) {
     if (region->regiontype == RGN_TYPE_WINDOW) {
@@ -994,13 +997,14 @@ static void view3d_localview_exit(const Depsgraph *depsgraph,
         continue;
       }
 
-      if (frame_selected) {
+      if (frame_selected && depsgraph) {
         Object *camera_old_rv3d, *camera_new_rv3d;
 
         camera_old_rv3d = (rv3d->persp == RV3D_CAMOB) ? camera_old : nullptr;
         camera_new_rv3d = (rv3d->localvd->persp == RV3D_CAMOB) ? camera_new : nullptr;
 
         rv3d->view = rv3d->localvd->view;
+        rv3d->view_axis_roll = rv3d->localvd->view_axis_roll;
         rv3d->persp = rv3d->localvd->persp;
         rv3d->camzoom = rv3d->localvd->camzoom;
 
@@ -1019,11 +1023,40 @@ static void view3d_localview_exit(const Depsgraph *depsgraph,
 
       MEM_freeN(rv3d->localvd);
       rv3d->localvd = nullptr;
+      changed = true;
     }
   }
+  return changed;
 }
 
-static int localview_exec(bContext *C, wmOperator *op)
+bool ED_localview_exit_if_empty(const Depsgraph *depsgraph,
+                                Scene *scene,
+                                ViewLayer *view_layer,
+                                wmWindowManager *wm,
+                                wmWindow *win,
+                                View3D *v3d,
+                                ScrArea *area,
+                                const bool frame_selected,
+                                const int smooth_viewtx)
+{
+  if (v3d->localvd == nullptr) {
+    return false;
+  }
+
+  v3d->localvd->runtime.flag &= ~V3D_RUNTIME_LOCAL_MAYBE_EMPTY;
+
+  BKE_view_layer_synced_ensure(scene, view_layer);
+  LISTBASE_FOREACH (Base *, base, BKE_view_layer_object_bases_get(view_layer)) {
+    if (base->local_view_bits & v3d->local_view_uid) {
+      return false;
+    }
+  }
+
+  return view3d_localview_exit(
+      depsgraph, wm, win, scene, view_layer, area, frame_selected, smooth_viewtx);
+}
+
+static wmOperatorStatus localview_exec(bContext *C, wmOperator *op)
 {
   const Depsgraph *depsgraph = CTX_data_ensure_evaluated_depsgraph(C);
   const int smooth_viewtx = WM_operator_smooth_viewtx_get(op);
@@ -1038,9 +1071,8 @@ static int localview_exec(bContext *C, wmOperator *op)
   bool changed;
 
   if (v3d->localvd) {
-    view3d_localview_exit(
+    changed = view3d_localview_exit(
         depsgraph, wm, win, scene, view_layer, area, frame_selected, smooth_viewtx);
-    changed = true;
   }
   else {
     changed = view3d_localview_init(depsgraph,
@@ -1080,9 +1112,10 @@ void VIEW3D_OT_localview(wmOperatorType *ot)
   ot->description = "Toggle display of selected object(s) separately and centered in view";
   ot->idname = "VIEW3D_OT_localview";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = localview_exec;
-  ot->flag = OPTYPE_UNDO; /* localview changes object layer bitflags */
+  /* Use undo because local-view changes object layer bit-flags. */
+  ot->flag = OPTYPE_UNDO;
 
   ot->poll = ED_operator_view3d_active;
 
@@ -1093,7 +1126,7 @@ void VIEW3D_OT_localview(wmOperatorType *ot)
                   "Move the view to frame the selected objects");
 }
 
-static int localview_remove_from_exec(bContext *C, wmOperator *op)
+static wmOperatorStatus localview_remove_from_exec(bContext *C, wmOperator *op)
 {
   View3D *v3d = CTX_wm_view3d(C);
   Main *bmain = CTX_data_main(C);
@@ -1103,8 +1136,8 @@ static int localview_remove_from_exec(bContext *C, wmOperator *op)
   BKE_view_layer_synced_ensure(scene, view_layer);
   LISTBASE_FOREACH (Base *, base, BKE_view_layer_object_bases_get(view_layer)) {
     if (BASE_SELECTED(v3d, base)) {
-      base->local_view_bits &= ~v3d->local_view_uuid;
-      ED_object_base_select(base, BA_DESELECT);
+      base->local_view_bits &= ~v3d->local_view_uid;
+      blender::ed::object::base_select(base, blender::ed::object::BA_DESELECT);
 
       if (base == view_layer->basact) {
         view_layer->basact = nullptr;
@@ -1113,6 +1146,19 @@ static int localview_remove_from_exec(bContext *C, wmOperator *op)
     }
   }
 
+  /* If some object was removed from the local view, exit the local view if it is now empty. */
+  if (changed) {
+    ED_localview_exit_if_empty(CTX_data_ensure_evaluated_depsgraph(C),
+                               scene,
+                               view_layer,
+                               CTX_wm_manager(C),
+                               CTX_wm_window(C),
+                               v3d,
+                               CTX_wm_area(C),
+                               true,
+                               WM_operator_smooth_viewtx_get(op));
+  }
+
   if (changed) {
     DEG_tag_on_visible_update(bmain, false);
     DEG_id_tag_update(&scene->id, ID_RECALC_SELECT);
@@ -1142,7 +1188,7 @@ void VIEW3D_OT_localview_remove_from(wmOperatorType *ot)
   ot->description = "Move selected objects out of local view";
   ot->idname = "VIEW3D_OT_localview_remove_from";
 
-  /* api callbacks */
+  /* API callbacks. */
   ot->exec = localview_remove_from_exec;
   ot->poll = localview_remove_from_poll;
   ot->flag = OPTYPE_UNDO;
@@ -1154,7 +1200,9 @@ void VIEW3D_OT_localview_remove_from(wmOperatorType *ot)
 /** \name Local Collections
  * \{ */
 
-static uint free_localcollection_bit(Main *bmain, ushort local_collections_uuid, bool *r_reset)
+static uint free_localcollection_bit(const Main *bmain,
+                                     ushort local_collections_uid,
+                                     bool *r_reset)
 {
   ushort local_view_bits = 0;
 
@@ -1165,7 +1213,7 @@ static uint free_localcollection_bit(Main *bmain, ushort local_collections_uuid,
         if (sl->spacetype == SPACE_VIEW3D) {
           View3D *v3d = reinterpret_cast<View3D *>(sl);
           if (v3d->flag & V3D_LOCAL_COLLECTIONS) {
-            local_view_bits |= v3d->local_collections_uuid;
+            local_view_bits |= v3d->local_collections_uid;
           }
         }
       }
@@ -1173,8 +1221,8 @@ static uint free_localcollection_bit(Main *bmain, ushort local_collections_uuid,
   }
 
   /* First try to keep the old uuid. */
-  if (local_collections_uuid && ((local_collections_uuid & local_view_bits) == 0)) {
-    return local_collections_uuid;
+  if (local_collections_uid && ((local_collections_uid & local_view_bits) == 0)) {
+    return local_collections_uid;
   }
 
   /* Otherwise get the first free available. */
@@ -1203,7 +1251,7 @@ static void local_collections_reset_uuid(LayerCollection *layer_collection,
   }
 }
 
-static void view3d_local_collections_reset(Main *bmain, const uint local_view_bit)
+static void view3d_local_collections_reset(const Main *bmain, const uint local_view_bit)
 {
   LISTBASE_FOREACH (Scene *, scene, &bmain->scenes) {
     LISTBASE_FOREACH (ViewLayer *, view_layer, &scene->view_layers) {
@@ -1214,7 +1262,7 @@ static void view3d_local_collections_reset(Main *bmain, const uint local_view_bi
   }
 }
 
-bool ED_view3d_local_collections_set(Main *bmain, View3D *v3d)
+bool ED_view3d_local_collections_set(const Main *bmain, View3D *v3d)
 {
   if ((v3d->flag & V3D_LOCAL_COLLECTIONS) == 0) {
     return true;
@@ -1222,13 +1270,13 @@ bool ED_view3d_local_collections_set(Main *bmain, View3D *v3d)
 
   bool reset = false;
   v3d->flag &= ~V3D_LOCAL_COLLECTIONS;
-  uint local_view_bit = free_localcollection_bit(bmain, v3d->local_collections_uuid, &reset);
+  uint local_view_bit = free_localcollection_bit(bmain, v3d->local_collections_uid, &reset);
 
   if (local_view_bit == 0) {
     return false;
   }
 
-  v3d->local_collections_uuid = local_view_bit;
+  v3d->local_collections_uid = local_view_bit;
   v3d->flag |= V3D_LOCAL_COLLECTIONS;
 
   if (reset) {
@@ -1238,10 +1286,10 @@ bool ED_view3d_local_collections_set(Main *bmain, View3D *v3d)
   return true;
 }
 
-void ED_view3d_local_collections_reset(bContext *C, const bool reset_all)
+void ED_view3d_local_collections_reset(const bContext *C, const bool reset_all)
 {
   Main *bmain = CTX_data_main(C);
-  uint local_view_bit = ~(0);
+  uint local_view_bit = ~0;
   bool do_reset = false;
 
   /* Reset only the ones that are not in use. */
@@ -1250,9 +1298,9 @@ void ED_view3d_local_collections_reset(bContext *C, const bool reset_all)
       LISTBASE_FOREACH (SpaceLink *, sl, &area->spacedata) {
         if (sl->spacetype == SPACE_VIEW3D) {
           View3D *v3d = reinterpret_cast<View3D *>(sl);
-          if (v3d->local_collections_uuid) {
+          if (v3d->local_collections_uid) {
             if (v3d->flag & V3D_LOCAL_COLLECTIONS) {
-              local_view_bit &= ~v3d->local_collections_uuid;
+              local_view_bit &= ~v3d->local_collections_uid;
             }
             else {
               do_reset = true;
@@ -1266,10 +1314,10 @@ void ED_view3d_local_collections_reset(bContext *C, const bool reset_all)
   if (do_reset) {
     view3d_local_collections_reset(bmain, local_view_bit);
   }
-  else if (reset_all && (do_reset || (local_view_bit != ~(0)))) {
-    view3d_local_collections_reset(bmain, ~(0));
+  else if (reset_all && (do_reset || (local_view_bit != ~0))) {
+    view3d_local_collections_reset(bmain, ~0);
     View3D v3d = {};
-    v3d.local_collections_uuid = ~(0);
+    v3d.local_collections_uid = ~0;
     BKE_layer_collection_local_sync(CTX_data_scene(C), CTX_data_view_layer(C), &v3d);
     DEG_id_tag_update(&CTX_data_scene(C)->id, ID_RECALC_BASE_FLAGS);
   }
