@@ -11,9 +11,7 @@
 
 #include "Precision.h"
 
-#ifdef WITH_CXX_GUARDEDALLOC
-#  include "MEM_guardedalloc.h"
-#endif
+#include "MEM_guardedalloc.h"
 
 namespace Freestyle {
 
@@ -33,9 +31,7 @@ class PseudoNoise {
   static const uint NB_VALUE_NOISE = 512;
   static real _values[NB_VALUE_NOISE];
 
-#ifdef WITH_CXX_GUARDEDALLOC
   MEM_CXX_CLASS_ALLOC_FUNCS("Freestyle:PseudoNoise")
-#endif
 };
 
 } /* namespace Freestyle */
