@@ -22,10 +22,12 @@
 
 #include "IO_dupli_persistent_id.hh"
 
+#include "BLI_hash.hh"
+#include "BLI_map.hh"
+#include "BLI_set.hh"
+
 #include "DEG_depsgraph.hh"
 
-#include <map>
-#include <set>
 #include <string>
 
 struct Depsgraph;
@@ -83,6 +85,18 @@ struct HierarchyContext {
   /* When true this is duplisource object. This flag is used to identify instance prototypes. */
   bool is_duplisource;
 
+  /* This flag tells whether an object is a valid point instance of other objects.
+   * If true, it means the object has a valid reference path and its value can be included
+   * in the instances data of UsdGeomPointInstancer. */
+  bool is_point_instance;
+
+  /* This flag tells if an object is a valid prototype of a point instancer. */
+  bool is_point_proto;
+
+  /* True if this context is a descendant of any context with is_point_instance set to true.
+   * This helps skip redundant instancing data during export. */
+  bool has_point_instance_ancestor;
+
   /*********** Determined during writer creation: ***************/
   float parent_matrix_inv_world[4][4]; /* Inverse of the parent's world matrix. */
   std::string export_path; /* Hierarchical path, such as "/grandparent/parent/object_name". */
@@ -101,8 +115,6 @@ struct HierarchyContext {
    * refers to a different object). */
   std::string higher_up_export_path;
 
-  bool operator<(const HierarchyContext &other) const;
-
   /* Return a HierarchyContext representing the root of the export hierarchy. */
   static const HierarchyContext *root();
 
@@ -112,6 +124,9 @@ struct HierarchyContext {
   void mark_as_not_instanced();
   bool is_prototype() const;
 
+  /* For handling point instancing (Instance on Points geometry node). */
+  bool is_point_instancer() const;
+
   bool is_object_visible(enum eEvaluationMode evaluation_mode) const;
 };
 
@@ -194,9 +209,13 @@ class ObjectIdentifier {
                                                 Object *duplicated_by);
 
   bool is_root() const;
+
+  uint64_t hash() const
+  {
+    return get_default_hash(object, duplicated_by, persistent_id);
+  }
 };
 
-bool operator<(const ObjectIdentifier &obj_ident_a, const ObjectIdentifier &obj_ident_b);
 bool operator==(const ObjectIdentifier &obj_ident_a, const ObjectIdentifier &obj_ident_b);
 
 /* AbstractHierarchyIterator iterates over objects in a dependency graph, and constructs export
@@ -209,16 +228,19 @@ bool operator==(const ObjectIdentifier &obj_ident_a, const ObjectIdentifier &obj
 class AbstractHierarchyIterator {
  public:
   /* Mapping from export path to writer. */
-  using WriterMap = std::map<std::string, AbstractHierarchyWriter *>;
+  using WriterMap = blender::Map<std::string, AbstractHierarchyWriter *>;
   /* All the children of some object, as per the export hierarchy. */
-  using ExportChildren = std::set<HierarchyContext *>;
+  using ExportChildren = blender::Set<HierarchyContext *>;
   /* Mapping from an object and its duplicator to the object's export-children. */
-  using ExportGraph = std::map<ObjectIdentifier, ExportChildren>;
+  using ExportGraph = blender::Map<ObjectIdentifier, ExportChildren>;
   /* Mapping from ID to its export path. This is used for instancing; given an
    * instanced datablock, the export path of the original can be looked up. */
-  using ExportPathMap = std::map<ID *, std::string>;
+  using ExportPathMap = blender::Map<ID *, std::string>;
+  /* Mapping from ID name to a set of names logically residing "under" it. Used for unique
+   * name generation. */
+  using ExportUsedNameMap = blender::Map<std::string, blender::Set<std::string>>;
   /* IDs of all duplisource objects, used to identify instance prototypes. */
-  using DupliSources = std::set<ID *>;
+  using DupliSources = blender::Set<ID *>;
 
  protected:
   ExportGraph export_graph_;
@@ -228,6 +250,7 @@ class AbstractHierarchyIterator {
   WriterMap writers_;
   ExportSubset export_subset_;
   DupliSources duplisources_;
+  ExportUsedNameMap used_names_;
 
  public:
   explicit AbstractHierarchyIterator(Main *bmain, Depsgraph *depsgraph);
@@ -254,6 +277,9 @@ class AbstractHierarchyIterator {
    * This base implementation is a no-op; override in a concrete subclass. */
   virtual std::string make_valid_name(const std::string &name) const;
 
+  virtual std::string make_unique_name(const std::string &original_name,
+                                       Set<std::string> &used_names);
+
   /* Return the name of this ID datablock that is valid for the exported file format. Overriding is
    * only necessary if make_valid_name(id->name+2) is not suitable for the exported file format.
    * NULL-safe: when `id == nullptr` this returns an empty string. */
@@ -278,7 +304,7 @@ class AbstractHierarchyIterator {
                           const DupliParentFinder &dupli_parent_finder);
 
   void context_update_for_graph_index(HierarchyContext *context,
-                                      const ExportGraph::key_type &graph_index) const;
+                                      const ObjectIdentifier &graph_index) const;
 
   void determine_export_paths(const HierarchyContext *parent_context);
   bool determine_duplication_references(const HierarchyContext *parent_context,
@@ -294,7 +320,8 @@ class AbstractHierarchyIterator {
   HierarchyContext context_for_object_data(const HierarchyContext *object_context) const;
 
   /* Convenience wrappers around get_id_name(). */
-  std::string get_object_name(const Object *object) const;
+  std::string get_object_name(const Object *object);
+  std::string get_object_name(const Object *object, const Object *parent);
   std::string get_object_data_name(const Object *object) const;
 
   using create_writer_func =
@@ -328,8 +355,8 @@ class AbstractHierarchyIterator {
 
   virtual bool should_visit_dupli_object(const DupliObject *dupli_object) const;
 
-  virtual ExportGraph::key_type determine_graph_index_object(const HierarchyContext *context);
-  virtual ExportGraph::key_type determine_graph_index_dupli(
+  virtual ObjectIdentifier determine_graph_index_object(const HierarchyContext *context);
+  virtual ObjectIdentifier determine_graph_index_dupli(
       const HierarchyContext *context,
       const DupliObject *dupli_object,
       const DupliParentFinder &dupli_parent_finder);
@@ -364,7 +391,7 @@ class AbstractHierarchyIterator {
   }
 
   AbstractHierarchyWriter *get_writer(const std::string &export_path) const;
-  ExportChildren &graph_children(const HierarchyContext *context);
+  ExportChildren *graph_children(const HierarchyContext *context);
 };
 
 }  // namespace blender::io
