@@ -8,10 +8,6 @@
 
 #pragma once
 
-#ifdef __cplusplus
-extern "C" {
-#endif
-
 struct ClothModifierData;
 struct Depsgraph;
 struct Implicit_Data;
@@ -40,7 +36,3 @@ int SIM_cloth_solve(struct Depsgraph *depsgraph,
                     struct ListBase *effectors);
 void SIM_cloth_solver_set_positions(struct ClothModifierData *clmd);
 void SIM_cloth_solver_set_volume(struct ClothModifierData *clmd);
-
-#ifdef __cplusplus
-}
-#endif
